#ifndef DDR3_MAP_H_
#define DDR3_MAP_H_

#include <inttypes.h>
#include "acq_chan.h"

/*************** Memory Regions *************/
#define MEM_TOTAL_SIZE (1 << 28) // 256 MB
#define MEM_REGION_SIZE (MEM_TOTAL_SIZE / 8)

/* FPGA Specific */
/* Do not touch these unless you know what you are doing! */
#define DDR3_PAYLOAD_SIZE 32 /* In Bytes: Virtex6 (ML605) and Artix7 (AFC) */

/* ADC */
#define DDR3_ADC_SAMPLE_SIZE ADC_SAMPLE_SIZE
#define DDR3_ADC_START_ADDR 0x0
#define DDR3_ADC_END_ADDR (DDR3_ADC_START_ADDR + 2*MEM_REGION_SIZE - DDR3_ADC_SAMPLE_SIZE)
#define DDR3_ADC_MAX_SAMPLES ((DDR3_ADC_END_ADDR-DDR3_ADC_START_ADDR) / DDR3_ADC_SAMPLE_SIZE)

/* TBT AMP */
#define DDR3_TBTAMP_SAMPLE_SIZE TBTAMP_SAMPLE_SIZE
#define DDR3_TBTAMP_START_ADDR (DDR3_ADC_END_ADDR + DDR3_ADC_SAMPLE_SIZE)
#define DDR3_TBTAMP_END_ADDR (DDR3_TBTAMP_START_ADDR + MEM_REGION_SIZE - DDR3_TBTAMP_SAMPLE_SIZE)
#define DDR3_TBTAMP_MAX_SAMPLES ((DDR3_TBTAMP_END_ADDR-DDR3_TBTAMP_START_ADDR) / DDR3_TBTAMP_SAMPLE_SIZE)

// TBT POS
#define DDR3_TBTPOS_SAMPLE_SIZE TBTPOS_SAMPLE_SIZE
#define DDR3_TBTPOS_START_ADDR (DDR3_TBTAMP_END_ADDR + DDR3_TBTAMP_SAMPLE_SIZE)
#define DDR3_TBTPOS_END_ADDR (DDR3_TBTPOS_START_ADDR + 2*MEM_REGION_SIZE - DDR3_TBTPOS_SAMPLE_SIZE)
#define DDR3_TBTPOS_MAX_SAMPLES ((DDR3_TBTPOS_END_ADDR-DDR3_TBTPOS_START_ADDR) / DDR3_TBTPOS_SAMPLE_SIZE)

// FOFB AMP
#define DDR3_FOFBAMP_SAMPLE_SIZE FOFBAMP_SAMPLE_SIZE
#define DDR3_FOFBAMP_START_ADDR (DDR3_TBTPOS_END_ADDR + DDR3_TBTPOS_SAMPLE_SIZE)
#define DDR3_FOFBAMP_END_ADDR (DDR3_FOFBAMP_START_ADDR + MEM_REGION_SIZE - DDR3_FOFBAMP_SAMPLE_SIZE)
#define DDR3_FOFBAMP_MAX_SAMPLES ((DDR3_FOFBAMP_END_ADDR-DDR3_FOFBAMP_START_ADDR) / DDR3_FOFBAMP_SAMPLE_SIZE)

// FOFB POS
#define DDR3_FOFBPOS_SAMPLE_SIZE FOFBPOS_SAMPLE_SIZE
#define DDR3_FOFBPOS_START_ADDR (DDR3_FOFBAMP_END_ADDR + DDR3_FOFBAMP_SAMPLE_SIZE)
#define DDR3_FOFBPOS_END_ADDR (DDR3_FOFBPOS_START_ADDR + 2*MEM_REGION_SIZE - DDR3_FOFBPOS_SAMPLE_SIZE)
#define DDR3_FOFBPOS_MAX_SAMPLES ((DDR3_FOFBPOS_END_ADDR-DDR3_FOFBPOS_START_ADDR) / DDR3_FOFBPOS_SAMPLE_SIZE)

struct _acq_buf_t {
  uint32_t id;
  uint32_t start_addr;
  uint32_t end_addr;
  uint32_t max_samples;
  uint32_t sample_size;
};

typedef struct _acq_buf_t acq_buf_t;

extern const acq_buf_t __acq_buf[END_CHAN_ID];

#endif

